@@ -1,7 +1,7 @@
 /*
  *  handle.c
  *
- *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -81,7 +81,9 @@ void _alpm_handle_free(alpm_handle_t *handle)
 	_alpm_trans_free(handle->trans);
 	FREE(handle->root);
 	FREE(handle->dbpath);
+	FREE(handle->dbext);
 	FREELIST(handle->cachedirs);
+	FREELIST(handle->hookdirs);
 	FREE(handle->logfile);
 	FREE(handle->lockfile);
 	FREE(handle->arch);
@@ -124,16 +126,30 @@ int _alpm_handle_lock(alpm_handle_t *handle)
 	return (handle->lockfd >= 0 ? 0 : -1);
 }
 
-/** Remove a lock file */
-int _alpm_handle_unlock(alpm_handle_t *handle)
+/** Remove the database lock file
+ * @param handle the context handle
+ * @return 0 on success, -1 on error
+ *
+ * @note Safe to call from inside signal handlers.
+ */
+int SYMEXPORT alpm_unlock(alpm_handle_t *handle)
 {
-	ASSERT(handle->lockfile != NULL, return -1);
+	ASSERT(handle->lockfile != NULL, return 0);
 	ASSERT(handle->lockfd >= 0, return 0);
 
 	close(handle->lockfd);
 	handle->lockfd = -1;
 
 	if(unlink(handle->lockfile) != 0) {
+		RET_ERR(handle, ALPM_ERR_SYSTEM, -1);
+	} else {
+		return 0;
+	}
+}
+
+int _alpm_handle_unlock(alpm_handle_t *handle)
+{
+	if(alpm_unlock(handle) != 0) {
 		if(errno == ENOENT) {
 			_alpm_log(handle, ALPM_LOG_WARNING,
 					_("lock file missing %s\n"), handle->lockfile);
@@ -148,6 +164,7 @@ int _alpm_handle_unlock(alpm_handle_t *handle)
 			return -1;
 		}
 	}
+
 	return 0;
 }
 
@@ -206,6 +223,12 @@ const char SYMEXPORT *alpm_option_get_dbpath(alpm_handle_t *handle)
 	return handle->dbpath;
 }
 
+alpm_list_t SYMEXPORT *alpm_option_get_hookdirs(alpm_handle_t *handle)
+{
+	CHECK_HANDLE(handle, return NULL);
+	return handle->hookdirs;
+}
+
 alpm_list_t SYMEXPORT *alpm_option_get_cachedirs(alpm_handle_t *handle)
 {
 	CHECK_HANDLE(handle, return NULL);
@@ -284,6 +307,12 @@ int SYMEXPORT alpm_option_get_checkspace(alpm_handle_t *handle)
 	return handle->checkspace;
 }
 
+const char SYMEXPORT *alpm_option_get_dbext(alpm_handle_t *handle)
+{
+	CHECK_HANDLE(handle, return NULL);
+	return handle->dbext;
+}
+
 int SYMEXPORT alpm_option_set_logcb(alpm_handle_t *handle, alpm_cb_log cb)
 {
 	CHECK_HANDLE(handle, return -1);
@@ -380,6 +409,58 @@ alpm_errno_t _alpm_set_directory_option(const char *value,
 	return 0;
 }
 
+int SYMEXPORT alpm_option_add_hookdir(alpm_handle_t *handle, const char *hookdir)
+{
+	char *newhookdir;
+
+	CHECK_HANDLE(handle, return -1);
+	ASSERT(hookdir != NULL, RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1));
+
+	newhookdir = canonicalize_path(hookdir);
+	if(!newhookdir) {
+		RET_ERR(handle, ALPM_ERR_MEMORY, -1);
+	}
+	handle->hookdirs = alpm_list_add(handle->hookdirs, newhookdir);
+	_alpm_log(handle, ALPM_LOG_DEBUG, "option 'hookdir' = %s\n", newhookdir);
+	return 0;
+}
+
+int SYMEXPORT alpm_option_set_hookdirs(alpm_handle_t *handle, alpm_list_t *hookdirs)
+{
+	alpm_list_t *i;
+	CHECK_HANDLE(handle, return -1);
+	if(handle->hookdirs) {
+		FREELIST(handle->hookdirs);
+	}
+	for(i = hookdirs; i; i = i->next) {
+		int ret = alpm_option_add_hookdir(handle, i->data);
+		if(ret) {
+			return ret;
+		}
+	}
+	return 0;
+}
+
+int SYMEXPORT alpm_option_remove_hookdir(alpm_handle_t *handle, const char *hookdir)
+{
+	char *vdata = NULL;
+	char *newhookdir;
+	CHECK_HANDLE(handle, return -1);
+	ASSERT(hookdir != NULL, RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1));
+
+	newhookdir = canonicalize_path(hookdir);
+	if(!newhookdir) {
+		RET_ERR(handle, ALPM_ERR_MEMORY, -1);
+	}
+	handle->hookdirs = alpm_list_remove_str(handle->hookdirs, newhookdir, &vdata);
+	FREE(newhookdir);
+	if(vdata != NULL) {
+		FREE(vdata);
+		return 1;
+	}
+	return 0;
+}
+
 int SYMEXPORT alpm_option_add_cachedir(alpm_handle_t *handle, const char *cachedir)
 {
 	char *newcachedir;
@@ -584,9 +665,15 @@ int SYMEXPORT alpm_option_remove_ignoregroup(alpm_handle_t *handle, const char *
 
 int SYMEXPORT alpm_option_add_assumeinstalled(alpm_handle_t *handle, const alpm_depend_t *dep)
 {
+	alpm_depend_t *depcpy;
 	CHECK_HANDLE(handle, return -1);
+	ASSERT(dep->mod == ALPM_DEP_MOD_EQ || dep->mod == ALPM_DEP_MOD_ANY,
+			RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1));
+	ASSERT((depcpy = _alpm_dep_dup(dep)), RET_ERR(handle, ALPM_ERR_MEMORY, -1));
 
-	handle->assumeinstalled = alpm_list_add(handle->assumeinstalled, (void *)dep);
+	/* fill in name_hash in case dep was built by hand */
+	depcpy->name_hash = _alpm_hash_sdbm(dep->name);
+	handle->assumeinstalled = alpm_list_add(handle->assumeinstalled, depcpy);
 	return 0;
 }
 
@@ -597,7 +684,12 @@ int SYMEXPORT alpm_option_set_assumeinstalled(alpm_handle_t *handle, alpm_list_t
 		alpm_list_free_inner(handle->assumeinstalled, (alpm_list_fn_free)alpm_dep_free);
 		alpm_list_free(handle->assumeinstalled);
 	}
-	handle->assumeinstalled = deps;
+	while(deps) {
+		if(alpm_option_add_assumeinstalled(handle, deps->data) != 0) {
+			return -1;
+		}
+		deps = deps->next;
+	}
 	return 0;
 }
 
@@ -606,10 +698,21 @@ static int assumeinstalled_cmp(const void *d1, const void *d2)
 	const alpm_depend_t *dep1 = d1;
 	const alpm_depend_t *dep2 = d2;
 
-	if(strcmp(dep1->name, dep2->name) == 0 && strcmp(dep1->version, dep2->version) == 0) {
+	if(dep1->name_hash != dep2->name_hash
+			|| strcmp(dep1->name, dep2->name) != 0) {
+		return -1;
+	}
+
+	if(dep1->version && dep2->version
+			&& strcmp(dep1->version, dep2->version) == 0) {
 		return 0;
 	}
 
+	if(dep1->version == NULL && dep2->version == NULL) {
+		return 0;
+	}
+
+
 	return -1;
 }
 
@@ -664,6 +767,21 @@ int SYMEXPORT alpm_option_set_checkspace(alpm_handle_t *handle, int checkspace)
 	return 0;
 }
 
+int SYMEXPORT alpm_option_set_dbext(alpm_handle_t *handle, const char *dbext)
+{
+	CHECK_HANDLE(handle, return -1);
+	ASSERT(dbext, RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1));
+
+	if(handle->dbext) {
+		FREE(handle->dbext);
+	}
+
+	STRDUP(handle->dbext, dbext, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
+
+	_alpm_log(handle, ALPM_LOG_DEBUG, "option 'dbext' = %s\n", handle->dbext);
+	return 0;
+}
+
 int SYMEXPORT alpm_option_set_default_siglevel(alpm_handle_t *handle,
 		alpm_siglevel_t level)
 {
