@@ -1,7 +1,7 @@
 /*
  *  callback.c
  *
- *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -99,7 +99,7 @@ static void fill_progress(const int bar_percent, const int disp_percent,
 		const int proglen)
 {
 	/* 8 = 1 space + 1 [ + 1 ] + 5 for percent */
-	const int hashlen = proglen - 8;
+	const int hashlen = proglen > 8 ? proglen - 8 : 0;
 	const int hash = bar_percent * hashlen / 100;
 	static int lasthash = 0, mouth = 0;
 	int i;
@@ -160,6 +160,16 @@ static void fill_progress(const int bar_percent, const int disp_percent,
 	fflush(stdout);
 }
 
+static int number_length(size_t n)
+{
+	int digits = 1;
+	while((n /= 10)) {
+		++digits;
+	}
+
+	return digits;
+}
+
 /* callback to handle messages/notifications from libalpm transactions */
 void cb_event(alpm_event_t *event)
 {
@@ -167,6 +177,22 @@ void cb_event(alpm_event_t *event)
 		return;
 	}
 	switch(event->type) {
+		case ALPM_EVENT_HOOK_START:
+			if(event->hook.when == ALPM_HOOK_PRE_TRANSACTION) {
+				colon_printf(_("Running pre-transaction hooks...\n"));
+			} else {
+				colon_printf(_("Running post-transaction hooks...\n"));
+			}
+			break;
+		case ALPM_EVENT_HOOK_RUN_START:
+			{
+				alpm_event_hook_run_t *e = &event->hook_run;
+				int digits = number_length(e->total);
+				printf("(%*zu/%*zu) %s\n", digits, e->position,
+						digits, e->total, 
+						e->desc ? e->desc : e->name);
+			}
+			break;
 		case ALPM_EVENT_CHECKDEPS_START:
 			printf(_("checking dependencies...\n"));
 			break;
@@ -181,6 +207,9 @@ void cb_event(alpm_event_t *event)
 		case ALPM_EVENT_INTERCONFLICTS_START:
 			printf(_("looking for conflicting packages...\n"));
 			break;
+		case ALPM_EVENT_TRANSACTION_START:
+			colon_printf(_("Processing package changes...\n"));
+			break;
 		case ALPM_EVENT_PACKAGE_OPERATION_START:
 			if(config->noprogressbar) {
 				alpm_event_package_operation_t *e = &event->package_operation;
@@ -269,9 +298,11 @@ void cb_event(alpm_event_t *event)
 		case ALPM_EVENT_OPTDEP_REMOVAL:
 			{
 				alpm_event_optdep_removal_t *e = &event->optdep_removal;
+				char *dep_string = alpm_dep_compute_string(e->optdep);
 				colon_printf(_("%s optionally requires %s\n"),
 						alpm_pkg_get_name(e->pkg),
-						alpm_dep_compute_string(e->optdep));
+						dep_string);
+				free(dep_string);
 			}
 			break;
 		case ALPM_EVENT_DATABASE_MISSING:
@@ -313,27 +344,12 @@ void cb_event(alpm_event_t *event)
 				}
 			}
 			break;
-		case ALPM_EVENT_PACORIG_CREATED:
-			{
-				alpm_event_pacorig_created_t *e = &event->pacorig_created;
-				if(on_progress) {
-					char *string = NULL;
-					pm_sprintf(&string, ALPM_LOG_WARNING, _("%s saved as %s.pacorig\n"),
-							e->file, e->file);
-					if(string != NULL) {
-						output = alpm_list_add(output, string);
-					}
-				} else {
-					pm_printf(ALPM_LOG_WARNING, _("%s saved as %s.pacorig\n"),
-							e->file, e->file);
-				}
-			}
-			break;
 		/* all the simple done events, with fallthrough for each */
 		case ALPM_EVENT_FILECONFLICTS_DONE:
 		case ALPM_EVENT_CHECKDEPS_DONE:
 		case ALPM_EVENT_RESOLVEDEPS_DONE:
 		case ALPM_EVENT_INTERCONFLICTS_DONE:
+		case ALPM_EVENT_TRANSACTION_DONE:
 		case ALPM_EVENT_INTEGRITY_DONE:
 		case ALPM_EVENT_KEYRING_DONE:
 		case ALPM_EVENT_KEY_DOWNLOAD_DONE:
@@ -343,6 +359,8 @@ void cb_event(alpm_event_t *event)
 		case ALPM_EVENT_DISKSPACE_DONE:
 		case ALPM_EVENT_RETRIEVE_DONE:
 		case ALPM_EVENT_RETRIEVE_FAILED:
+		case ALPM_EVENT_HOOK_DONE:
+		case ALPM_EVENT_HOOK_RUN_DONE:
 		/* we can safely ignore those as well */
 		case ALPM_EVENT_PKGDOWNLOAD_START:
 		case ALPM_EVENT_PKGDOWNLOAD_DONE:
@@ -432,8 +450,9 @@ void cb_question(alpm_question_t *question)
 				alpm_question_select_provider_t *q = &question->select_provider;
 				size_t count = alpm_list_count(q->providers);
 				char *depstring = alpm_dep_compute_string(q->depend);
-				colon_printf(_("There are %zd providers available for %s:\n"), count,
-						depstring);
+				colon_printf(_n("There is %zu provider available for %s\n",
+						"There are %zu providers available for %s:\n", count),
+						count, depstring);
 				free(depstring);
 				select_display(q->providers);
 				q->use_index = select_question(count);
@@ -456,10 +475,10 @@ void cb_question(alpm_question_t *question)
 				strftime(created, 12, "%Y-%m-%d", localtime(&time));
 
 				if(q->key->revoked) {
-					q->import = yesno(_("Import PGP key %d%c/%s, \"%s\", created: %s (revoked)?"),
+					q->import = yesno(_("Import PGP key %u%c/%s, \"%s\", created: %s (revoked)?"),
 							q->key->length, q->key->pubkey_algo, q->key->fingerprint, q->key->uid, created);
 				} else {
-					q->import = yesno(_("Import PGP key %d%c/%s, \"%s\", created: %s?"),
+					q->import = yesno(_("Import PGP key %u%c/%s, \"%s\", created: %s?"),
 							q->key->length, q->key->pubkey_algo, q->key->fingerprint, q->key->uid, created);
 				}
 			}
@@ -482,7 +501,6 @@ void cb_progress(alpm_progress_t event, const char *pkgname, int percent,
 	/* size of line to allocate for text printing (e.g. not progressbar) */
 	int infolen;
 	int digits, textlen;
-	size_t tmp;
 	char *opr = NULL;
 	/* used for wide character width determination and printing */
 	int len, wclen, wcwid, padwid;
@@ -558,11 +576,8 @@ void cb_progress(alpm_progress_t event, const char *pkgname, int percent,
 	}
 
 	/* find # of digits in package counts to scale output */
-	digits = 1;
-	tmp = howmany;
-	while((tmp /= 10)) {
-		++digits;
-	}
+	digits = number_length(howmany);
+
 	/* determine room left for non-digits text [not ( 1/12) part] */
 	textlen = infolen - 3 /* (/) */ - (2 * digits) - 1 /* space */;
 
@@ -593,7 +608,7 @@ void cb_progress(alpm_progress_t event, const char *pkgname, int percent,
 		int i = textlen - 3;
 		wchar_t *p = wcstr;
 		/* grab the max number of char columns we can fill */
-		while(i > 0 && wcwidth(*p) < i) {
+		while(i - wcwidth(*p) > 0) {
 			i -= wcwidth(*p);
 			p++;
 		}
@@ -603,8 +618,8 @@ void cb_progress(alpm_progress_t event, const char *pkgname, int percent,
 
 	}
 
-	printf("(%*ld/%*ld) %ls%-*s", digits, (unsigned long)current,
-			digits, (unsigned long)howmany, wcstr, padwid, "");
+	printf("(%*zu/%*zu) %ls%-*s", digits, current,
+			digits, howmany, wcstr, padwid, "");
 
 	free(wcstr);
 
@@ -763,7 +778,7 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 	fname = malloc(len + 1);
 	memcpy(fname, filename, len);
 	/* strip package or DB extension for cleaner look */
-	if((p = strstr(fname, ".pkg")) || (p = strstr(fname, ".db"))) {
+	if((p = strstr(fname, ".pkg")) || (p = strstr(fname, ".db")) || (p = strstr(fname, ".files"))) {
 		/* tack on a .sig suffix for signatures */
 		if(memcmp(&filename[len - 4], ".sig", 4) == 0) {
 			memcpy(p, ".sig", 4);
