@@ -1,5 +1,5 @@
 #  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
-#  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
+#  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -174,8 +174,7 @@ class pmpkg(object):
     def install_package(self, root):
         """Install the package in the given root."""
         for f in self.files:
-            util.mkfile(root, f, f)
-            path = os.path.join(root, f)
+            path = util.mkfile(root, f, f)
             if os.path.isfile(path):
                 os.utime(path, (355, 355))
 
